#Imports
import numpy as np
import cv2
def plot_boxes(pred_class, pred_boxes,image, path, image_name, color = None):
  arr = image.astype('float32')
  arr = cv2.cvtColor(arr, cv2.COLOR_RGB2BGR)
  colorsarr = []
  if(color==None): 
    colorsarr =[(1.0,0,1.0),(0,1.0,1.0),(1.0,0,0),(0,0,1.0),(0,1.0,0)]
  else:
    colorsarr = [color,color,color,color,color]
  for idx,box in enumerate(pred_boxes):
    if(idx>=5):
      break
    arr = cv2.rectangle(arr, box[0], box[1], colorsarr[idx], 2)
    cv2.putText(arr, pred_class[idx], (box[0][0], max(box[0][1],20)), cv2.FONT_HERSHEY_DUPLEX, 1, colorsarr[idx], 2)
  cv2.imwrite(str(path+'/'+image_name+'.jpg'), 255*arr)
  return cv2.cvtColor(arr, cv2.COLOR_BGR2RGB)
  #cv2.imshow('Image',arr)
  #cv2.waitKey(100)
   # Write the required arguments

  # The function should plot the predicted boxes on the images and save them.
  # Tip: keep the dimensions of the output image less than 800 to avoid RAM crashes.
  
  


#Imports
import numpy as np

class CropImage(object):
    '''
        Performs either random cropping or center cropping.
    '''

    def __init__(self, shape, crop_type='center'):
        self.crop_type = crop_type
        self.shape = shape
        '''
            Arguments:
            shape: output shape of the crop (h, w)
            crop_type: center crop or random crop. Default: center
        '''
    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
        y,x,_ = image.shape
        cropx = self.shape[1]
        cropy = self.shape[0]
        if(self.crop_type == 'center'):
            startx = x//2-(cropx//2)
            starty = y//2-(cropy//2)    
            return image[starty:starty+cropy,startx:startx+cropx]
        elif(self.crop_type == 'random'):
            startx = np.random.randint(0,x-cropx+1)
            starty = np.random.randint(0,y-cropy+1)
            return image[starty:starty+cropy,startx:startx+cropx]



#Imports
import numpy as np
import json
from matplotlib import image
from matplotlib import pyplot
from pathlib import Path

class Dataset(object):
    '''
        A class for the dataset that will return data items as per the given index
    '''

    def __init__(self, annotation_file, transforms = None):
        self.annotation_file = annotation_file
        self.transforms = transforms
        self.datalist = []
        with open(self.annotation_file) as f:
            for line in f:
                data = json.loads(line)
                gt_bboxes = np.array([])
                for box in data['bboxes']:
                    added = np.append(box['category_id'], box['bbox'])
                    if(len(gt_bboxes) == 0):
                        gt_bboxes = added
                        gt_bboxes = gt_bboxes.reshape(1,5)
                    else:
                        gt_bboxes = np.vstack((gt_bboxes,added))
                p = Path(annotation_file)
                dir_data = str(p.parent.absolute())
                image1 = image.imread(dir_data + '\\' + data['img_fn'])
                image1 = np.interp(image1, (image1.min(), image1.max()), (0, +1))
                if(self.transforms is not None):
                    for t in self.transforms:
                        image1 = t(image1)
                image1 = np.transpose(image1, (2,0,1))
                self.datalist.append([image1, gt_bboxes])
                
        '''
            Arguments:
            annotation_file: path to the annotation file
            transforms: list of transforms (class instances)
                        For instance, [<class 'RandomCrop'>, <class 'Rotate'>]
        '''

        

    def __len__(self):
        return(len(self.datalist))
        '''
            return the number of data points in the dataset
        '''

        

    def __getitem__(self, idx):
        d= {}
        d["annotations"] = self.datalist[idx][1]
        d["image"] = self.datalist[idx][0]
        return d
        
        '''
            return the dataset element for the index: "idx"
            Arguments:
                idx: index of the data element.

            Returns: A dictionary with:
                image: image (in the form of a numpy array) (shape: (3, H, W))
                gt_bboxes: N X 5 array where N is the number of bounding boxes, each 
                            consisting of [class, x1, y1, x2, y2]
                            x1 and x2 lie between 0 and width of the image,
                            y1 and y2 lie between 0 and height of the image.

            You need to do the following, 
            1. Extract the correct annotation using the idx provided.
            2. Read the image and convert it into a numpy array (wont be necessary
                with some libraries). The shape of the array would be (3, H, W).
            3. Scale the values in the array to be with [0, 1].
            4. Create a dictonary with both the image and annotations
            4. Perform the desired transformations.
            5. Return the transformed image and annotations as specified. '''



#Imports
import scipy
from scipy import ndimage
import numpy as np
class RotateImage(object):
    '''
        Rotates the image about the centre of the image.
    '''

    def __init__(self, degrees):
        '''
            Arguments:
            degrees: rotation degree.
        '''
        self.degrees = degrees
        # Write your code here

    def __call__(self, sample):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
        return scipy.ndimage.rotate(sample, self.degrees)



#Imports
import numpy as np

class FlipImage(object):
    '''
        Flips the image.
    '''
    def __init__(self, flip_type='horizontal'):
        '''
            Arguments:
            flip_type: 'horizontal' or 'vertical' Default: 'horizontal'
        '''
        self.type = flip_type
        # Write your code here

        
    def __call__(self, image):
        if(self.type == 'horizontal'):
            return np.flip(image, axis = 0)
        elif(self.type == 'vertical'):
            return np.flip(image, axis = 1)
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
